function [data, Routput, Rchadj, Rtadj, Regstack] = register_FISSEQ_images_131213(input_dir,output_dir)

%
% Validate Input
%

% Initialize timer
tic;

% Check that the input directory exists
if exist(input_dir) ~= 7
    error('Specified Input Directory does not exist')
end

% Create the output directory if it does not exist
if exist(output_dir) == 0
    mkdir(output_dir)
end

% Find .ics files in the directory
files = dir(input_dir);
ics_files = regexp({files.name},'/*.ics','match');
w_ics_files = find(~cellfun(@isempty,ics_files));

% Report time
disp('Initialize:');
toc;

%
% Initialize data array
%

% Initialize timer
tic;

% Preallocate storage to keep track
dathold{length(w_ics_files)+1} = 'storage';
where_dathold{length(w_ics_files)+1} = 'where_storage';
hold = struct([]);

% Process the first file in the list and pre-allocate storage
im = bfopen(strcat(input_dir, '/',files(w_ics_files(1)).name));
stacks = size(im{1},1);
[d1,d2] = size(im{1}{1,1});
sizedat = strsplit(im{1}{1,2},{'/',';','=','_'});
data = zeros([d1,d2,5,length(w_ics_files)]);
im_data = zeros(d1,d2,str2num(sizedat{24}),5);
for j = 1:stacks
    locdat = strsplit(im{1}{j,2},{'/',';','=','_'});
    im_data(:,:,str2num(locdat{23}),str2num(locdat{26})) = im{1}{j,1};
    hold(1).zw(j) = str2num(locdat{23});
    hold(1).cw(j) = str2num(locdat{26});
end
im_data(:,:,:,5) = max(im_data(:,:,:,1:4),[],4);
dathold{1} = squeeze(max(im_data,[],3));
where_dathold{1} = str2num(sizedat{10});

% Import image stack for each timepoint
parfor i = 2:length(w_ics_files)
    im = bfopen(strcat(input_dir, '/',files(w_ics_files(i)).name));
    stacks = size(im{1},1);
    sizedat = strsplit(im{1}{1,2},{'/',';','=','_'});
    im_data = zeros(d1,d2,str2num(sizedat{24}),5);
    for j = 1:stacks
        locdat = strsplit(im{1}{j,2},{'/',';','=','_'});
        im_data(:,:,str2num(locdat{23}),str2num(locdat{26})) = im{1}{j,1};
        hold(i).zw(j) = str2num(locdat{23});
        hold(i).cw(j) = str2num(locdat{26});
    end
    im_data(:,:,:,5) = max(im_data(:,:,:,1:4),[],4);
    dathold{i} = squeeze(max(im_data,[],3));
    where_dathold{i} = str2num(sizedat{10});
end

% Assemble images from container
for i = 1:length(w_ics_files)
    data(:,:,:,where_dathold{i}) = dathold{i};
end

% Clear memory hog storage container
clear dathold

% Report time
disp('Import & Store All Images:');
toc;

%
% Image Registration
%

% Initialize timer
tic;

% Process Alignment
[Routput, Rchadj, Rtadj, Regstack] = FISSEQ_dftregistration_chadjust_blockreg2(data,10,10,0.1);

% Write Tiff Images to file
for t = 1:size(Regstack,4)
    for ch = 1:size(Regstack,3)
        fn = sprintf('%sf3_T_%02d_C_%02d.tif',output_dir,t-1,ch-1);
        imwrite(im2uint8(mat2gray(Regstack(:,:,ch,t))),fn,'tiff');
    end
end

% Save alignment parameters
fn_Rout = sprintf('%soutput.m',output_dir);
fn_Rchadj = sprintf('%schadj.m',output_dir);
fn_Rtadj = sprintf('%stadj.m',output_dir);
save(fn_Rout,'Routput');
save(fn_Rchadj,'Rchadj');
save(fn_Rtadj,'Rtadj');

% Report time
disp('Register Images:');
toc;

return
